import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Form, Row, Col, Input, Select, InputNumber, Radio, Tooltip, Modal, notification, Popover } from 'antd'
import { QuestionCircleOutlined } from '@ant-design/icons'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import { getColumnForm } from './formconfig'
import { formRule } from '@/utils/option.js'
import CodeMirror from '@/templates/zshare/codemirror'
import EditTable from '@/templates/zshare/modalform/modaleditable'
import './index.scss'


const { TextArea } = Input
const columnTypeOptions = {
  text: ['label', 'field', 'type', 'Align', 'Hide', 'IsSort', 'Width', 'prefix', 'postfix', 'textFormat', 'editable', 'initval', 'blacklist'],
  number: ['label', 'field', 'type', 'Align', 'Hide', 'IsSort', 'Width', 'decimal', 'format', 'prefix', 'postfix', 'editable', 'initval', 'sum', 'blacklist'],
  textarea: ['label', 'field', 'type', 'Align', 'Hide', 'Width', 'prefix', 'initval', 'postfix', 'blacklist'],
  custom: ['label', 'type', 'Align', 'Width', 'blacklist'],
  colspan: ['label', 'type', 'Align', 'Hide', 'blacklist'],
  action: ['label', 'type', 'Align', 'Width'],
  formula: ['label', 'type', 'Align', 'Hide', 'Width', 'prefix', 'postfix', 'eval', 'formula', 'blacklist'],
  index: ['label', 'type', 'Align', 'Width']
}

class EdiTableColumn extends Component {
  static propTpyes = {
    column: PropTypes.object,
    columns: PropTypes.array,
    fields: PropTypes.array,
    submitCol: PropTypes.func,  // 提交事件
    cancelCol: PropTypes.func   // 取消时删除事件
  }

  state = {
    visible: false,
    loading: false,
    formlist: null,
    transfield: {}
  }

  record = null

  UNSAFE_componentWillMount() {
    let transfield = {}
    this.props.columns.forEach(item => {
      transfield[item.field] = item.label
    })

    this.setState({transfield})
  }

  UNSAFE_componentWillReceiveProps (nextProps) {
    if (nextProps.column && !is(fromJS(this.props.column), fromJS(nextProps.column))) {
      this.editColumn(nextProps.column)
    }
  }

  getOptions = () => {
    let _options = fromJS(columnTypeOptions[this.record.type]).toJS()

    if (['number', 'text'].includes(this.record.type) && this.record.editable === 'true') {
      _options.push('ctrlField')
      if (this.record.ctrlField) {
        _options.push('ctrlValue')
      }
      if (this.record.type === 'text') {
        _options.push('editType')

        if (this.record.editType === 'switch') {
          _options.push('enter', 'openVal', 'closeVal', 'openText', 'closeText', 'editField')
        } else if (this.record.editType === 'select') {
          _options.push('required', 'enter', 'resourceType', 'linkSubField', 'editField', 'dropdown')

          if (this.record.resourceType === '0') {
            _options.push('options')
          } else {
            _options.push('dataSource', 'valueField', 'valueText', 'orderBy', 'orderType', 'disableField', 'database')
          }
        } else {
          _options.push('required', 'enter')
        }
      } else if (this.record.type === 'number') {
        _options.push('max', 'min', 'enter')
      }
    }
    if (this.record.type === 'formula' && this.record.eval === 'true') {
      _options.push('decimal')
    }
    if (['number', 'formula'].includes(this.record.type) && this.record.Hide !== 'true') {
      _options.push('noValue')
    }

    return _options
  }

  editColumn = (column) => {
    let fields = fromJS(this.props.fields).toJS().map(item => {
      item.text = `${item.field}（${item.label}）`
      return item
    })

    let formlist = getColumnForm(column, fields, this.props.columns)
    this.record = {}

    formlist.forEach(item => {
      this.record[item.key] = item.initVal
    })
    
    let _options = this.getOptions()

    this.setState({
      visible: true,
      formlist: formlist.map(item => {
        item.hidden = !_options.includes(item.key)

        if (item.key === 'formula') {
          item.fields = this.props.fields.map(col => col.field)
          item.fields = item.fields.join(', ')
        }

        return item
      })
    })
    if (column.focus) {
      setTimeout(() => {
        try {
          let _form = document.getElementById('label')
          _form && _form.select()
        } catch (e) {
          console.warn('表单focus失败！')
        }
      }, 200)
    }
  }

  typeChange = (key, value, option) => {
    this.record[key] = value

    if (key === 'type') {
      let _options = this.getOptions()

      let _field = ''
      if (value === 'formula') {
        _field = this.props.form.getFieldValue('field') || ''
      }

      this.setState({
        formlist: this.state.formlist.map(item => {
          if (item.key === 'decimal' && value === 'formula') {
            this.record.decimal = ''
          }

          item.initVal = this.record[item.key]
          item.hidden = !_options.includes(item.key)

          return item
        })
      }, () => {
        if (value === 'colspan') {
          this.props.form.setFieldsValue({Align: 'center'})
        } else if (value === 'formula' && _field) {
          this.props.form.setFieldsValue({formula: '@' + _field + '@'})
        } else if (value === 'index') {
          this.props.form.setFieldsValue({label: '序号'})
        }
      })
    } else if (key === 'field') {
      let values = {label: option.props.label || option.props.children}
      if (/Decimal|int/ig.test(option.props.datatype)) {
        let decimal = 0
        if (/Decimal/ig.test(option.props.datatype)) {
          decimal = +option.props.datatype.replace(/Decimal\(18,/ig, '').replace(')', '')
        }
        values.type = 'number'
        values.decimal = decimal
      } else {
        values.type = 'text'
      }

      let _type = this.record.type
      this.record.type = values.type

      if (values.type !== _type) {
        let _options = this.getOptions()

        this.setState({
          formlist: this.state.formlist.map(item => {
            item.initVal = this.record[item.key]
            item.hidden = !_options.includes(item.key)

            return item
          })
        }, () => {
          this.props.form.setFieldsValue(values)
        })
      } else {
        this.props.form.setFieldsValue(values)
      }
    } else if (key === 'format' && value === 'percent') {
      this.props.form.setFieldsValue({postfix: '%'})
    } else if (['editable', 'editType', 'resourceType', 'ctrlField', 'eval', 'Hide'].includes(key)) {
      let _options = this.getOptions()

      this.setState({
        formlist: this.state.formlist.map(item => {
          item.initVal = this.record[item.key]
          item.hidden = !_options.includes(item.key)

          return item
        })
      })
    }
  }

  multiselectChange = (key, value) => {
    if (key !== 'linkSubField') return

    this.record[key] = value
  }

  handleEmpty = () => {
    let field = this.props.form.getFieldValue('valueField')

    if (!field) {
      notification.warning({
        top: 92,
        message: '请填写值·字段。',
        duration: 5
      })
      return
    }

    let text = this.props.form.getFieldValue('valueText')

    if (!text) {
      notification.warning({
        top: 92,
        message: '请填写文本·字段。',
        duration: 5
      })
      return
    }

    let resource = this.props.form.getFieldValue('dataSource') || ''

    if (field === text) {
      resource = `select '' as ${field} union all \n${resource}`
    } else {
      resource = `select '' as ${field},'全部' as ${text} union all \n${resource}`
    }

    this.props.form.setFieldsValue({dataSource: resource})
  }

  changeOptions = (data) => {
    this.record.options = data || []
  }

  getFields() {
    const { getFieldDecorator } = this.props.form
    const { formlist, transfield } = this.state
    const fields = []

    if (!formlist) return null

    formlist.forEach((item, index) => {
      if (item.hidden || item.forbidden) return

      let span = 12
      let rules = []
      let className = ''
      let content = null
      let extra = null
      let initVal = item.initVal || ''

      if (item.type === 'text') {
        rules = [
          { required: item.required, message: '请输入' + item.label + '!' },
          {
            max: formRule.input.max,
            message: formRule.input.message
          }
        ]

        content = <Input placeholder="" autoComplete="off" onPressEnter={this.handleSubmit} />
      } else if (item.type === 'number') {
        rules = [
          { required: item.required, message: '请输入' + item.label + '!' }
        ]
        initVal = item.initVal

        if (!item.unlimit) {
          content = <InputNumber min={item.min} max={item.max} precision={item.precision} onPressEnter={this.handleSubmit}/>
        } else {
          content = <InputNumber onPressEnter={this.handleSubmit}/>
        }
      } else if (item.type === 'select') {
        rules = [
          { required: item.required, message: '请选择' + item.label + '!' }
        ]
        content = <Select
          showSearch
          allowClear={item.allowClear === true}
          filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
          onChange={(value, option) => {this.typeChange(item.key, value, option)}}
          getPopupContainer={() => document.getElementById('edit-table-column-winter')}
        >
          {item.options.map((option, i) =>
            <Select.Option key={i} datatype={option.datatype || ''} label={option.label || ''} value={(option.value || option.field || option.MenuID)}>
              {(option.text || option.label || option.MenuName)}
            </Select.Option>
          )}
        </Select>
      } else if (item.type === 'radio') {
        rules = [
          { required: item.required, message: '请选择' + item.label + '!' }
        ]
        initVal = item.initVal

        content = <Radio.Group onChange={(e) => {this.typeChange(item.key, e.target.value)}}>
          {item.options.map(option => <Radio key={option.value} value={option.value}>{option.text}</Radio>)}
        </Radio.Group>
      } else if (item.type === 'multiselect') { // 多选
        content = <Select
          showSearch
          mode="multiple"
          filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
          onChange={(value) => this.multiselectChange(item.key, value)}
        >
          {item.options.map((option, i) =>
            <Select.Option key={i} value={option.value || option.field}>{option.text || option.label}</Select.Option>
          )}
        </Select>
      } else if (item.type === 'textarea') {
        span = 24
        className = 'text-area'
        rules = [
          { required: item.required, message: '请输入' + item.label + '!' }
        ]

        if (item.key === 'formula') {
          fields.push(
            <Col span={span} key={index}>
              <Form.Item className={className} extra={extra} label={item.tooltip ?
                <Tooltip placement="topLeft" title={item.tooltip}>
                  <QuestionCircleOutlined className="mk-form-tip" />
                  {item.label}
                </Tooltip> : item.label
              }>
                {getFieldDecorator(item.key, {
                  initialValue: initVal,
                  rules: rules
                })(<TextArea rows={item.rows || 2}/>)}
              </Form.Item>
              <Popover overlayClassName="formula-fields" placement="topLeft" title="" content={<div>{item.fields}</div>} trigger="click">
                <span className="formula-icon">字段集</span>
              </Popover>
            </Col>
          )
          return
        }

        content = <TextArea rows={item.rows || 2}/>
      } else if (item.type === 'codemirror') {
        rules = [
          { required: item.required, message: '请输入' + item.label + '!' }
        ]
        span = 24
        className = 'text-area'

        extra = <span className="add-resource-empty" onClick={this.handleEmpty}>空</span>
        if (item.placeholder) {
          extra = <><span className="resource-public-var">{item.placeholder}</span>{extra}</>
        }

        content = <CodeMirror />
      } else if (item.type === 'options') {
        span = 24
        className = 'text-area'

        let linkSubFields = this.record.linkSubField || []

        let columns = []

        columns.push({ title: 'Value', key: 'Value', strict: true })
        columns.push({ title: 'Text', key: 'Text' })

        linkSubFields.forEach(field => {
          if (field === 'Value' || field === 'Text') return

          columns.push({ title: transfield[field] || field, key: field })
        })
      
        content = <EditTable columns={columns} module="form" onChange={this.changeOptions}/>
      }

      fields.push(
        <Col span={span} key={index}>
          <Form.Item className={className} extra={extra} label={item.tooltip ?
            <Tooltip placement="topLeft" title={item.tooltip}>
              <QuestionCircleOutlined className="mk-form-tip" />
              {item.label}
            </Tooltip> : item.label
          }>
            {getFieldDecorator(item.key, {
              initialValue: initVal,
              rules: rules
            })(content)}
          </Form.Item>
        </Col>
      )
    })
    return fields
  }

  transfer = (options) => {
    if (options.length === 0) return options

    let isNumber = true
    options.forEach(item => {
      if (!item.Value || isNaN(item.Value)) {
        isNumber = false
      }
    })

    if (isNumber) {
      return options.map(item => {
        item.Value = +item.Value
        return item
      })
    } else {
      return options.map(item => {
        item.Value = item.Value + ''
        return item
      })
    }
  }

  handleSubmit = () => {
    const { fields } = this.props
    // 表单提交时检查输入值是否正确
    this.props.form.validateFieldsAndScroll((err, values) => {
      if (!err) {
        if (values.type === 'number' && values.editable === 'true') {
          if (typeof(values.max) === 'number' && typeof(values.min) === 'number' && values.max < values.min) {
            notification.warning({
              top: 92,
              message: '最大值不可小于最小值！',
              duration: 5
            })
            return
          }
          // eslint-disable-next-line
        } else if (values.type === 'formula' && values.eval === 'true' && /^[\u4E00-\u9FA50-9a-zA-Z_\s@\+\-\*\/]*$/ig.test(values.formula) && /[\+\-\*\/]/ig.test(values.formula)) {
          let cols = []
          fields.forEach(col => {
            if (/^(Int|Decimal)/ig.test(col.datatype)) {
              cols.push({reg: new RegExp('@' + col.field + '@', 'ig'), value: `(@${col.field}@)`})
            }
          })

          cols.forEach(col => {
            values.formula = values.formula.replace(col.reg, col.value)
          })
        } else if (values.type === 'text' && values.editable === 'true' && values.editType === 'select') {
          if (values.resourceType === '0') {
            values.options = values.options || []
            
            values.options = this.transfer(values.options)

            if (values.options.filter(op => op.Text === '').length > 0) {
              notification.warning({
                top: 92,
                message: '提示文本（Text）不可为空！',
                duration: 5
              })
              return
            } else {
              let arr = values.options.map(m => m.Value)
              let _arr = Array.from(new Set(arr))
              if (arr.length > _arr.length) {
                notification.warning({
                  top: 92,
                  message: 'Value值不可重复！',
                  duration: 5
                })
                return
              }
            }
          }
        }

        if (values.dataSource && /\s/.test(values.dataSource)) {
          let error = Utils.verifySql(values.dataSource)

          if (error) {
            notification.warning({
              top: 92,
              message: '数据源中不可使用' + error,
              duration: 5
            })
            return
          }

          this.setState({
            loading: true
          })

          let sql = `declare @mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20)
          ${values.dataSource}`

          sql = sql.replace(/@\$|\$@/ig, '').replace(/@(BID|ID|LoginUID|SessionUid|UserID|Appkey|time_id)@/ig, `'1949-10-01 15:00:00'`)

          let rduri = ''
          if (window.GLOB.mainSystemApi && values.database === 'sso') {
            rduri = window.GLOB.mainSystemApi
          }
          
          Api.sDebug(sql, rduri).then(result => {
            if (result.status || result.ErrCode === '-2') {
              this.setState({visible: false, loading: false, formlist: null})
              this.props.submitCol(values)
              this.record = null
            } else {
              this.setState({loading: false})
              Modal.error({
                title: result.message
              })
            }
          })
        } else {
          this.setState({visible: false, formlist: null})
          this.props.submitCol(values)
          this.record = null
        }
      }
    })
  }

  editModalCancel = () => {
    this.setState({visible: false, loading: false, formlist: null})

    this.props.cancelCol()
  }

  render() {
    const { visible, loading } = this.state
    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 6 }
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 18 }
      }
    }

    return (
      <div style={{display: 'inline-block'}}>
        <Modal
          title="显示列编辑"
          wrapClassName="mk-scroll-modal"
          visible={visible}
          width={900}
          maskClosable={false}
          onOk={this.handleSubmit}
          onCancel={this.editModalCancel}
          confirmLoading={loading}
          destroyOnClose
        >
          <Form {...formItemLayout} className="commontable-column-form" id="edit-table-column-winter">
            <Row gutter={24}>{this.getFields()}</Row>
          </Form>
        </Modal>
      </div>
    )
  }
}

export default Form.create()(EdiTableColumn)